/*
 * @Author: hai.zhang
 * @LastEditors: hai.zhang
 * @Date: 2019-07-15 16:40:10
 * @LastEditTime: 2022-01-05 10:22:17
 */
const fs = require('fs')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// 文件或目录是否存在
const exists = exports.exists = function (path) {
    return fs.existsSync(path)
}

// 是否是文件
exports.isFile = function (path) {
    return exists(path) && fs.statSync(path).isFile()
}

// 是否是目录
exports.isDir = function (path) {
    return exists(path) && fs.statSync(path).isDirectory()
}


exports.stlyeRules = function stlyeRules (sourceMap, extract, userConfig) {
    // const types = [
    //   { test: /\.css$/, loader: 'css' },
    //   { test: /\.styl$/, loader: 'stylus' },
    //   { test: /\.less/, loader: 'less' },
    //   { test: /\.s[ac]ss$/, loader: 'sass' }]
      
    const types = [
        {
          test: /\.css$/,
          use: [ ]
        },
        {
          test: /\.(sc|sa)ss$/,
          use: [ {
            loader: 'sass-loader', 
            options: {
              implementation: require('sass')
            },
          }, {
            loader: 'sass-resources-loader',
            options: {
              // Or array of paths
              resources: [path.resolve('common/css/variable.scss')]
            }
          }]
        },
        {
          test: /\.less$/,
          use: [ 'less-loader']
        },
        {
          test: /\.stylus$/,
          use: [ 'stylus-loader']
        }
      ]
    const options = { sourceMap: !!sourceMap }
    const use = [
      {
        loader: MiniCssExtractPlugin.loader,
        options: {}
      },
      {
        loader: 'css-loader',
        options: options
      },
      {
        loader: 'postcss-loader'
      }
    ]
    if (userConfig.device && userConfig.device === 'mobile') {
      use.push({
        loader: 'px2rem-loader',
        options: {
          remUnit: 75,
          remPrecision: 8
        }
      })
    }

    return types.map((t) => {
      const tuse = t.loader === 'css' ? use : use.concat({
        loader: t.loader + '-loader',
        options: options
      })
      if (t.loader === 'sass' && userConfig.sassResources) {
        tuse.push({
          loader: 'sass-resources-loader',
          options: {
            // Or array of paths
            resources: userConfig.sassResources
          }
        })
      }
      return extract ? {
        test: t.test,
        use: tuse
      } : {
        test: t.test,
        use: tuse
      }
    })
  }