/*
 * @Author: hai.zhang
 * @LastEditors: hai.zhang
 * @Date: 2019-07-15 16:40:10
 * @LastEditTime: 2022-01-05 15:21:01
 */
const fs = require('fs')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// 文件或目录是否存在
const exists = exports.exists = function (path) {
    return fs.existsSync(path)
}

// 是否是文件
exports.isFile = function (path) {
    return exists(path) && fs.statSync(path).isFile()
}

// 是否是目录
exports.isDir = function (path) {
    return exists(path) && fs.statSync(path).isDirectory()
}


exports.stlyeRules = function stlyeRules (sourceMap, userConfig) {
    const options = { sourceMap: !!sourceMap }
    
    const types = [
      {
        test: /\.css$/,
        use: []
      },
      {
        test: /\.(sc|sa)ss$/,
        use: [ 
          {
            loader: 'sass-loader', 
            options: Object.assign({
              implementation: require('sass'),
              options: options
            }, options),
          }, 
          {
            loader: 'sass-resources-loader',
            options: {
              // Or array of paths
              resources: userConfig.sassResources
            }
          }
        ]
      },
      {
        test: /\.less$/,
        use: [ { 
          loader: 'less-loader',
          options:  Object.assign({}, options)
        } ]
      },
      {
        test: /\.stylus$/,
        use: [ { 
          loader: 'stylus-loader',
          options:  Object.assign({}, options)
        } ]
      }
    ]
    const use = [
      {
        loader: MiniCssExtractPlugin.loader,
        options: {}
      },
      {
        loader: 'css-loader',
        options: options
      },
      {
        loader: 'postcss-loader'
      }
    ]
    if (userConfig.device && userConfig.device === 'mobile') {
      use.push({
        loader: 'px2rem-loader',
        options: {
          remUnit: 75,
          remPrecision: 8
        }
      })
    }
    return types.map((t) => {
      t.use = t.use.concat(use)
      return t
    })
  }