const path = require("path")
const webpack = require("webpack")
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const SpeedMeasurePlugin = require("speed-measure-webpack-plugin");
const merge = require('webpack-merge')
const util = require("../util")

const MiniCssExtractPlugin = require('mini-css-extract-plugin')

// const OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin')
const  CssMinimizerPlugin  =  require ( "css-minimizer-webpack-plugin" ) ;

const HtmlWebpackPlugin = require('html-webpack-plugin')
const { userConfig, templates, webpackConfig } = require("./webpack.config.base")
const context = webpackConfig.context
let publicPath = userConfig.publicPath
const smp = new SpeedMeasurePlugin();
// 默认自己公司的配置
if (!publicPath) {
  let publicDomain = '//static.sunyur.com/'
  publicPath = publicDomain + path.basename(context).replace('style-', '') + '/'
} 

const userConfigWebpack = userConfig && userConfig.webpackConfig && userConfig.webpackConfig.prod || {}
// 异步模块打包 @todo 临时解决方案
const basePath = templates[0].basePath
let config = merge(webpackConfig, {
  output: Object.assign({}, webpackConfig.output, {
    // 输出的文件名 hash统一生成,chunkhash变化生成
    filename: '[name].js',
    // chunkFilename: basePath + '/[id].[chunkhash:8].js'
    chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].js' :basePath + '/[name].[chunkhash:8].js'
  }),
  module: {
    rules: [
      ...util.stlyeRules(userConfig["devtool"], userConfig)
    ]
  },
  cache: {
    type: 'filesystem', // 使用文件缓存
  },
  performance: {
    maxEntrypointSize: 5*1024*1024,
    maxAssetSize: 5*1024*1024
  },
  mode: 'production',
  plugins: [
    // 配置环境
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"'
      },
      IS_DEV: JSON.stringify(false)
    }),
    new VueLoaderPlugin(),
    // 对css压缩
    new webpack.LoaderOptionsPlugin({
      minimize: true
    }),

    // 当模块内容没有修改时，保持module.id不变
    // new webpack.HashedModuleIdsPlugin()

  ],
  optimization: {
    nodeEnv: 'production',
    // splitChunks: {
    //   // include all types of chunks
    //   chunks: 'all',
    //   // 重复打包问题
    //   cacheGroups:{
    //     vendors:{ // node_modules里的代码
    //       test: /[\\/]node_modules[\\/]/,
    //       chunks: "all",
    //       name: function (module, chunks, cacheGroupKey) {
    //         console.log(cacheGroupKey)
    //         return parseInt(Math.random() * 1000)
    //       }, //一定不要定义固定的name
    //       priority: 10, // 优先级
    //       enforce: true 
    //     }
    //   }
    // },
    moduleIds: 'deterministic',
    minimize: true,
    concatenateModules: true,
    minimizer: [
      new CssMinimizerPlugin({
        parallel: true
      }),
    ]
  },
  // 上线设置source-map会影响加载速度,但便于线上bug查找
  devtool: userConfig["devtool"],

  //编译期间一旦出错就停止编译
  bail: true,
  stats: 'errors-only',
}, userConfigWebpack)


// 提取文件的公共部分
if (userConfig && userConfig.vendor) {
  var filename = userConfig.chunkhash ? 'common.[chunkhash:' + userConfig.chunkhash + '].js' : 'common.js'
  templates.forEach((tpl, index) => {
    const common = tpl.basePath + '/common'
    const templateName = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename: templateName,
      template: tpl.template,
      chunks: [common, tpl.chunk],
      minify: {
        keepClosingSlash: true,
        removeAttributeQuotes: false,
        quoteCharacter: '"',
      },
      xhtml: false,
      inject: true
    }))
  })
  
} else{
  templates.forEach((tpl) => {
    const filename = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename,
      template: tpl.template,
      chunks: [tpl.chunk],
      inject: true
    }))
  })
}

if (userConfig.extractCSS) {
  config.plugins.push(
    // 提取css,生成对应对文件
    new MiniCssExtractPlugin({
      filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].css' : '[name].css',
      chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].css' :basePath + '/[name].[chunkhash:8].css',
      ignoreOrder: true
      // filename: '/[name].css',
    }),
  )
}


if (userConfig.bundleAnalyzerReport) {
  var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
  config.plugins.push(new BundleAnalyzerPlugin())
}

module.exports = config
